@@ -5,11 +5,10 @@ except BaseException:
 
 
 class ExceptionsTest(g.unittest.TestCase):
-
     def test_module(self):
         # create an ExceptionWrapper
         try:
-            raise ValueError('nah')
+            raise ValueError("nah")
         except BaseException as E:
             em = g.trimesh.exceptions.ExceptionWrapper(E)
 
@@ -21,12 +20,12 @@ class ExceptionsTest(g.unittest.TestCase):
             # should re-raise `ValueError('nah')`
             em.hi()
             # if we're here raise an error we don't catch
-            raise NameError('should not have worked!!')
+            raise NameError("should not have worked!!")
         except ValueError:
             # should have re-raised ValueError
             pass
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     g.trimesh.util.attach_to_log()
     g.unittest.main()
