@@ -5,7 +5,6 @@ except BaseException:
 
 
 class NearestTest(g.unittest.TestCase):
-
     def test_naive(self):
         """
         Test the naive nearest point function
@@ -21,22 +20,21 @@ class NearestTest(g.unittest.TestCase):
         triangles = sphere.triangles  # NOQA
 
         # do the check
-        closest, distance, tid = g.trimesh.proximity.closest_point_naive(
-            sphere, points)
+        closest, distance, _tid = g.trimesh.proximity.closest_point_naive(sphere, points)
 
         # the distance from a sphere of radius 1.0 to a sphere of radius 2.0
         # should be pretty darn close to 1.0
-        assert (g.np.abs(distance - 1.0) < .01).all()
+        assert (g.np.abs(distance - 1.0) < 0.01).all()
 
         # the vector for the closest point should be the same as the vector
         # to the query point
         vector = g.trimesh.util.diagonal_dot(closest, points / 2.0)
-        assert (g.np.abs(vector - 1.0) < .01).all()
+        assert (g.np.abs(vector - 1.0) < 0.01).all()
 
     def test_helper(self):
         # just make sure the plumbing returns something
         for mesh in g.get_meshes(2):
-            points = (g.random((100, 3)) - .5) * 100
+            points = (g.random((100, 3)) - 0.5) * 100
 
             a = mesh.nearest.on_surface(points)
             assert a is not None
@@ -45,8 +43,10 @@ class NearestTest(g.unittest.TestCase):
             assert b is not None
 
     def test_nearest_naive(self):
-        funs = [g.trimesh.proximity.closest_point_naive,
-                g.trimesh.proximity.closest_point]
+        funs = [
+            g.trimesh.proximity.closest_point_naive,
+            g.trimesh.proximity.closest_point,
+        ]
 
         data_points = g.deque()
         data_dist = g.deque()
@@ -61,13 +61,10 @@ class NearestTest(g.unittest.TestCase):
         assert g.np.ptp(data_points, axis=0).max() < g.tol.merge
         assert g.np.ptp(data_dist, axis=0).max() < g.tol.merge
 
-        log_msg = '\n'.join("{}: {}s".format(i, j)
-                            for i, j in zip(
-            [i.__name__ for i in funs],
-            g.np.diff(tic)))
-        g.log.info(
-            'Compared the following nearest point functions:\n' +
-            log_msg)
+        log_msg = "\n".join(
+            f"{i}: {j}s" for i, j in zip([i.__name__ for i in funs], g.np.diff(tic))
+        )
+        g.log.info("Compared the following nearest point functions:\n" + log_msg)
 
     def check_nearest_point_function(self, fun):
         # def plot_tri(tri, color='g'):
@@ -94,31 +91,30 @@ class NearestTest(g.unittest.TestCase):
         # set the points up in space
         query[:, 2] = 10
         # a circle of points inside-ish the triangle
-        query = g.np.vstack((query, query * .1))
+        query = g.np.vstack((query, query * 0.1))
 
         # loop through each triangle
         for triangle in triangles:
             # create a mesh with one triangle
             mesh = g.Trimesh(**g.trimesh.triangles.to_kwargs([triangle]))
 
-            result, result_distance, result_tid = fun(mesh, query)
+            result, result_distance, _result_tid = fun(mesh, query)
 
             polygon = g.Polygon(triangle[:, 0:2])
             polygon_buffer = polygon.buffer(1e-5)
 
             # all of the points returned should be on the triangle we're
             # querying
-            assert all(polygon_buffer.intersects(
-                g.Point(i)) for i in result[:, 0:2])
+            assert all(polygon_buffer.intersects(g.Point(i)) for i in result[:, 0:2])
 
             # see what distance shapely thinks the nearest point
             # is for the 2D triangle and the query points
-            distance_shapely = g.np.array([polygon.distance(g.Point(i))
-                                           for i in query[:, :2]])
+            distance_shapely = g.np.array(
+                [polygon.distance(g.Point(i)) for i in query[:, :2]]
+            )
 
             # see what distance our function returned for the nearest point
-            distance_ours = ((query[:, :2] - result[:, :2])
-                             ** 2).sum(axis=1) ** .5
+            distance_ours = ((query[:, :2] - result[:, :2]) ** 2).sum(axis=1) ** 0.5
 
             # how far was our distance from the one shapely gave
             distance_test = g.np.abs(distance_shapely - distance_ours)  # NOQA
@@ -131,10 +127,9 @@ class NearestTest(g.unittest.TestCase):
         # any rigid transform
         # chop query off to same length as triangles
         assert len(query) > len(triangles)
-        query = query[:len(triangles)]
+        query = query[: len(triangles)]
         # run the closest point query as a corresponding query
-        close = g.trimesh.triangles.closest_point(triangles=triangles,
-                                                  points=query)
+        close = g.trimesh.triangles.closest_point(triangles=triangles, points=query)
         # distance between closest point and query point
         # this should stay the same regardless of frame
         distance = g.np.linalg.norm(close - query, axis=1)
@@ -142,11 +137,11 @@ class NearestTest(g.unittest.TestCase):
             # transform the query points
             points = g.trimesh.transform_points(query, T)
             # transform the triangles we're checking
-            tri = g.trimesh.transform_points(
-                triangles.reshape((-1, 3)), T).reshape((-1, 3, 3))
+            tri = g.trimesh.transform_points(triangles.reshape((-1, 3)), T).reshape(
+                (-1, 3, 3)
+            )
             # run the closest point check
-            check = g.trimesh.triangles.closest_point(triangles=tri,
-                                                      points=points)
+            check = g.trimesh.triangles.closest_point(triangles=tri, points=points)
             check_distance = g.np.linalg.norm(check - points, axis=1)
             # should be the same in any frame
             assert g.np.allclose(check_distance, distance)
@@ -164,7 +159,7 @@ class NearestTest(g.unittest.TestCase):
 
         # constructed so origin is inside but also coplanar with
         # the nearest face
-        mesh = g.get_mesh('origin_inside.STL')
+        mesh = g.get_mesh("origin_inside.STL")
 
         # origin should be inside, so distance should be positive
         distance = mesh.nearest.signed_distance([[0, 0, 0]])
@@ -176,13 +171,12 @@ class NearestTest(g.unittest.TestCase):
 
         # should be well outside the box and not coplanar with a face
         # so the signed distance should be negative
-        distance = mesh.nearest.signed_distance(
-            [mesh.bounds[0] + [100, 100, 100]])
+        distance = mesh.nearest.signed_distance([mesh.bounds[0] + [100, 100, 100]])
 
         assert distance[0] < 0.0
 
     def test_edge_case(self):
-        mesh = g.get_mesh('20mm-xyz-cube.stl')
+        mesh = g.get_mesh("20mm-xyz-cube.stl")
         assert (mesh.nearest.signed_distance([[-51, 4.7, -20.6]]) < 0.0).all()
 
     def test_acute_edge_case(self):
@@ -197,22 +191,21 @@ class NearestTest(g.unittest.TestCase):
         # -> take an even number of points
         n = 20
         n += n % 2
-        pts = g.np.transpose([g.np.zeros(n),
-                              g.np.ones(n),
-                              g.np.linspace(-1, 1, n)])
+        pts = g.np.transpose([g.np.zeros(n), g.np.ones(n), g.np.linspace(-1, 1, n)])
 
         # the faces facing the points should differ for first and second half of the set
         # check their indices for inequality
         faceIdxsA, faceIdxsB = g.np.split(mesh.nearest.on_surface(pts)[-1], 2)
-        assert (g.np.all(faceIdxsA == faceIdxsA[0]) and
-                g.np.all(faceIdxsB == faceIdxsB[0]) and
-                faceIdxsA[0] != faceIdxsB[0])
+        assert (
+            g.np.all(faceIdxsA == faceIdxsA[0])
+            and g.np.all(faceIdxsB == faceIdxsB[0])
+            and faceIdxsA[0] != faceIdxsB[0]
+        )
 
     def test_candidates(self):
         mesh = g.trimesh.creation.random_soup(2000)
         points = g.random((2000, 3))
-        g.trimesh.proximity.nearby_faces(
-            mesh=mesh, points=points)
+        g.trimesh.proximity.nearby_faces(mesh=mesh, points=points)
 
     def test_returns_correct_point_in_ambiguous_cases(self):
         mesh = g.trimesh.Trimesh(
@@ -237,12 +230,15 @@ class NearestTest(g.unittest.TestCase):
         # return correct values and ignore the unreferenced points
         query_point = [-1.0, -1.0, -1.0]
         mesh = g.trimesh.Trimesh(
-            vertices=[[1.0, 0.0, 0.0],
-                      [0.0, 1.0, 0.0],
-                      [0.0, 0.0, 1.0],
-                      [-0.5, -0.5, -0.5]],
+            vertices=[
+                [1.0, 0.0, 0.0],
+                [0.0, 1.0, 0.0],
+                [0.0, 0.0, 1.0],
+                [-0.5, -0.5, -0.5],
+            ],
             faces=[[0, 1, 2]],
-            process=False)
+            process=False,
+        )
 
         proximity_query = g.trimesh.proximity.ProximityQuery(mesh)
         q = proximity_query.on_surface([query_point])
@@ -250,6 +246,6 @@ class NearestTest(g.unittest.TestCase):
         assert all(len(i) == 1 for i in q)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     g.trimesh.util.attach_to_log()
     g.unittest.main()
