@@ -1,46 +1,43 @@
-import os
 import json
+import os
+
 import numpy as np
 
-from .. import util
-from .. import resolvers
-
+from .. import resolvers, util
 from ..base import Trimesh
+from ..exceptions import ExceptionWrapper
 from ..parent import Geometry
 from ..points import PointCloud
 from ..scene.scene import Scene, append_scenes
-from ..constants import log_time, log
-
+from ..typed import Dict, List, Loadable, Optional, Union
+from ..util import log, now
 from . import misc
-
-from .xyz import _xyz_loaders
-from .ply import _ply_loaders
-from .stl import _stl_loaders
+from .binvox import _binvox_loaders
+from .cascade import _cascade_loaders
 from .dae import _collada_loaders
+from .gltf import _gltf_loaders
+from .misc import _misc_loaders
 from .obj import _obj_loaders
 from .off import _off_loaders
-from .misc import _misc_loaders
-from .gltf import _gltf_loaders
-from .xaml import _xaml_loaders
-from .binvox import _binvox_loaders
-from .threemf import _three_loaders
-from .openctm import _ctm_loaders
+from .ply import _ply_loaders
+from .stl import _stl_loaders
 from .threedxml import _threedxml_loaders
-
+from .threemf import _three_loaders
+from .xaml import _xaml_loaders
+from .xyz import _xyz_loaders
 
 try:
     from ..path.exchange.load import load_path, path_formats
 except BaseException as E:
     # save a traceback to see why path didn't import
-    from ..exceptions import closure
-    load_path = closure(E)
+    load_path = ExceptionWrapper(E)
     # no path formats available
 
-    def path_formats():
+    def path_formats() -> set:
         return set()
 
 
-def mesh_formats():
+def mesh_formats() -> set:
     """
     Get a list of mesh formats available to load.
 
@@ -50,10 +47,11 @@ def mesh_formats():
       Extensions of available mesh loaders,
       i.e. 'stl', 'ply', etc.
     """
-    return set(mesh_loaders.keys())
+    # filter out exceptionmodule loaders
+    return {k for k, v in mesh_loaders.items() if not isinstance(v, ExceptionWrapper)}
 
 
-def available_formats():
+def available_formats() -> set:
     """
     Get a list of all available loaders
 
@@ -63,18 +61,20 @@ def available_formats():
         Extensions of available loaders
         i.e. 'stl', 'ply', 'dxf', etc.
     """
-    # set
     loaders = mesh_formats()
     loaders.update(path_formats())
     loaders.update(compressed_loaders.keys())
+
     return loaders
 
 
-def load(file_obj,
-         file_type=None,
-         resolver=None,
-         force=None,
-         **kwargs):
+def load(
+    file_obj: Loadable,
+    file_type: Optional[str] = None,
+    resolver: Union[resolvers.Resolver, Dict, None] = None,
+    force: Optional[str] = None,
+    **kwargs,
+) -> Union[Geometry, List[Geometry]]:
     """
     Load a mesh or vectorized path into objects like
     Trimesh, Path2D, Path3D, Scene
@@ -101,19 +101,17 @@ def load(file_obj,
     # check to see if we're trying to load something
     # that is already a native trimesh Geometry subclass
     if isinstance(file_obj, Geometry):
-        log.info('Load called on %s object, returning input',
-                 file_obj.__class__.__name__)
+        log.info("Load called on %s object, returning input", file_obj.__class__.__name__)
         return file_obj
 
     # parse the file arguments into clean loadable form
-    (file_obj,  # file- like object
-     file_type,  # str, what kind of file
-     metadata,  # dict, any metadata from file name
-     opened,    # bool, did we open the file ourselves
-     resolver   # object to load referenced resources
-     ) = parse_file_args(file_obj=file_obj,
-                         file_type=file_type,
-                         resolver=resolver)
+    (
+        file_obj,  # file- like object
+        file_type,  # str, what kind of file
+        metadata,  # dict, any metadata from file name
+        opened,  # bool, did we open the file ourselves
+        resolver,  # object to load referenced resources
+    ) = _parse_file_args(file_obj=file_obj, file_type=file_type, resolver=resolver)
 
     try:
         if isinstance(file_obj, dict):
@@ -122,34 +120,24 @@ def load(file_obj,
             loaded = load_kwargs(kwargs)
         elif file_type in path_formats():
             # path formats get loaded with path loader
-            loaded = load_path(file_obj,
-                               file_type=file_type,
-                               **kwargs)
+            loaded = load_path(file_obj, file_type=file_type, **kwargs)
         elif file_type in mesh_loaders:
             # mesh loaders use mesh loader
-            loaded = load_mesh(file_obj,
-                               file_type=file_type,
-                               resolver=resolver,
-                               **kwargs)
+            loaded = load_mesh(file_obj, file_type=file_type, resolver=resolver, **kwargs)
         elif file_type in compressed_loaders:
             # for archives, like ZIP files
-            loaded = load_compressed(file_obj,
-                                     file_type=file_type,
-                                     **kwargs)
+            loaded = load_compressed(file_obj, file_type=file_type, **kwargs)
         elif file_type in voxel_loaders:
             loaded = voxel_loaders[file_type](
-                file_obj,
-                file_type=file_type,
-                resolver=resolver,
-                **kwargs)
+                file_obj, file_type=file_type, resolver=resolver, **kwargs
+            )
         else:
-            if file_type in ['svg', 'dxf']:
+            if file_type in ["svg", "dxf"]:
                 # call the dummy function to raise the import error
                 # this prevents the exception from being super opaque
                 load_path()
             else:
-                raise ValueError('File type: %s not supported' %
-                                 file_type)
+                raise ValueError(f"File type: {file_type} not supported")
     finally:
         # close any opened files even if we crashed out
         if opened:
@@ -165,21 +153,22 @@ def load(file_obj,
         file_obj.close()
 
     # combine a scene into a single mesh
-    if force == 'mesh' and isinstance(loaded, Scene):
+    if force == "mesh" and isinstance(loaded, Scene):
         return util.concatenate(loaded.dump())
-    if force == 'scene' and not isinstance(loaded, Scene):
+    if force == "scene" and not isinstance(loaded, Scene):
         return Scene(loaded)
 
     return loaded
 
 
-@log_time
-def load_mesh(file_obj,
-              file_type=None,
-              resolver=None,
-              **kwargs):
+def load_mesh(
+    file_obj: Loadable,
+    file_type: Optional[str] = None,
+    resolver: Union[resolvers.Resolver, Dict, None] = None,
+    **kwargs,
+) -> Union[Geometry, List[Geometry]]:
     """
-    Load a mesh file into a Trimesh object
+    Load a mesh file into a Trimesh object.
 
     Parameters
     -----------
@@ -192,27 +181,25 @@ def load_mesh(file_obj,
 
     Returns
     ----------
-    mesh : trimesh.Trimesh or trimesh.Scene
-      Loaded geometry data
+    mesh
+      Loaded geometry data.
     """
 
     # parse the file arguments into clean loadable form
-    (file_obj,  # file- like object
-     file_type,  # str, what kind of file
-     metadata,  # dict, any metadata from file name
-     opened,    # bool, did we open the file ourselves
-     resolver   # object to load referenced resources
-     ) = parse_file_args(file_obj=file_obj,
-                         file_type=file_type,
-                         resolver=resolver)
+    (
+        file_obj,  # file-like object
+        file_type,  # str: what kind of file
+        metadata,  # dict: any metadata from file name
+        opened,  # bool: did we open the file ourselves
+        resolver,  # Resolver: to load referenced resources
+    ) = _parse_file_args(file_obj=file_obj, file_type=file_type, resolver=resolver)
 
     try:
         # make sure we keep passed kwargs to loader
         # but also make sure loader keys override passed keys
-        results = mesh_loaders[file_type](file_obj,
-                                          file_type=file_type,
-                                          resolver=resolver,
-                                          **kwargs)
+        loader = mesh_loaders[file_type]
+        tic = now()
+        results = loader(file_obj, file_type=file_type, resolver=resolver, **kwargs)
         if not isinstance(results, list):
             results = [results]
 
@@ -221,12 +208,13 @@ def load_mesh(file_obj,
             kwargs.update(result)
             loaded.append(load_kwargs(kwargs))
             loaded[-1].metadata.update(metadata)
+
+        # todo : remove this
         if len(loaded) == 1:
             loaded = loaded[0]
-        # show the repr for loaded
-        log.debug('loaded {} using {}'.format(
-            str(loaded),
-            mesh_loaders[file_type].__name__))
+
+        # show the repr for loaded, loader used, and time
+        log.debug(f"loaded {loaded!s} using `{loader.__name__}` in {now() - tic:0.4f}s")
     finally:
         # if we failed to load close file
         if opened:
@@ -235,11 +223,7 @@ def load_mesh(file_obj,
     return loaded
 
 
-def load_compressed(file_obj,
-                    file_type=None,
-                    resolver=None,
-                    mixed=False,
-                    **kwargs):
+def load_compressed(file_obj, file_type=None, resolver=None, mixed=False, **kwargs):
     """
     Given a compressed archive load all the geometry that
     we can from it.
@@ -261,19 +245,17 @@ def load_compressed(file_obj,
     """
 
     # parse the file arguments into clean loadable form
-    (file_obj,  # file- like object
-     file_type,  # str, what kind of file
-     metadata,  # dict, any metadata from file name
-     opened,    # bool, did we open the file ourselves
-     resolver   # object to load referenced resources
-     ) = parse_file_args(file_obj=file_obj,
-                         file_type=file_type,
-                         resolver=resolver)
+    (
+        file_obj,  # file- like object
+        file_type,  # str, what kind of file
+        metadata,  # dict, any metadata from file name
+        opened,  # bool, did we open the file ourselves
+        resolver,  # object to load referenced resources
+    ) = _parse_file_args(file_obj=file_obj, file_type=file_type, resolver=resolver)
 
     try:
         # a dict of 'name' : file-like object
-        files = util.decompress(file_obj=file_obj,
-                                file_type=file_type)
+        files = util.decompress(file_obj=file_obj, file_type=file_type)
         # store loaded geometries as a list
         geometries = []
 
@@ -281,18 +263,17 @@ def load_compressed(file_obj,
         resolver = resolvers.ZipResolver(files)
 
         # try to save the files with meaningful metadata
-        if 'file_path' in metadata:
-            archive_name = metadata['file_path']
+        if "file_path" in metadata:
+            archive_name = metadata["file_path"]
         else:
-            archive_name = 'archive'
+            archive_name = "archive"
 
         # populate our available formats
         if mixed:
             available = available_formats()
         else:
             # all types contained in ZIP archive
-            contains = set(util.split_extension(n).lower()
-                           for n in files.keys())
+            contains = {util.split_extension(n).lower() for n in files.keys()}
             # if there are no mesh formats available
             if contains.isdisjoint(mesh_formats()):
                 available = path_formats()
@@ -303,29 +284,31 @@ def load_compressed(file_obj,
         for name, data in files.items():
             try:
                 # only load formats that we support
-                compressed_type = util.split_extension(
-                    name).lower()
+                compressed_type = util.split_extension(name).lower()
 
                 # if file has metadata type include it
-                if compressed_type in 'yaml':
+                if compressed_type in "yaml":
                     import yaml
+
                     meta_archive[name] = yaml.safe_load(data)
-                elif compressed_type in 'json':
+                elif compressed_type in "json":
                     import json
+
                     meta_archive[name] = json.loads(data)
 
                 if compressed_type not in available:
                     # don't raise an exception, just try the next one
                     continue
                 # store the file name relative to the archive
-                metadata['file_name'] = (archive_name + '/' +
-                                         os.path.basename(name))
+                metadata["file_name"] = archive_name + "/" + os.path.basename(name)
                 # load the individual geometry
-                loaded = load(file_obj=data,
-                              file_type=compressed_type,
-                              resolver=resolver,
-                              metadata=metadata,
-                              **kwargs)
+                loaded = load(
+                    file_obj=data,
+                    file_type=compressed_type,
+                    resolver=resolver,
+                    metadata=metadata,
+                    **kwargs,
+                )
 
                 # some loaders return multiple geometries
                 if util.is_sequence(loaded):
@@ -335,8 +318,7 @@ def load_compressed(file_obj,
                     # if the loader has returned a single geometry
                     geometries.append(loaded)
             except BaseException:
-                log.debug('failed to load file in zip',
-                          exc_info=True)
+                log.debug("failed to load file in zip", exc_info=True)
 
     finally:
         # if we opened the file in this function
@@ -374,10 +356,12 @@ def load_remote(url, **kwargs):
       Loaded result
     """
     # import here to keep requirement soft
-    import requests
+    import httpx
 
     # download the mesh
-    response = requests.get(url)
+    response = httpx.get(url, follow_redirects=True)
+    response.raise_for_status()
+
     # wrap as file object
     file_obj = util.wrap_as_stream(response.content)
 
@@ -389,25 +373,23 @@ def load_remote(url, **kwargs):
         # will be wrong so try to clean up the URL
         # urllib is Python 3 only
         import urllib
+
         # remove the url-safe encoding then split off query params
-        file_type = urllib.parse.unquote(
-            url).split('?', 1)[0].split('/')[-1].strip()
+        file_type = urllib.parse.unquote(url).split("?", 1)[0].split("/")[-1].strip()
     except BaseException:
         # otherwise just use the last chunk of URL
-        file_type = url.split('/')[-1].split('?', 1)[0]
+        file_type = url.split("/")[-1].split("?", 1)[0]
 
     # actually load the data from the retrieved bytes
-    loaded = load(file_obj=file_obj,
-                  file_type=file_type,
-                  resolver=resolver,
-                  **kwargs)
+    loaded = load(file_obj=file_obj, file_type=file_type, resolver=resolver, **kwargs)
     return loaded
 
 
-def load_kwargs(*args, **kwargs):
+def load_kwargs(*args, **kwargs) -> Geometry:
     """
     Load geometry from a properly formatted dict or kwargs
     """
+
     def handle_scene():
         """
         Load a scene from our kwargs.
@@ -417,9 +399,8 @@ def load_kwargs(*args, **kwargs):
         graph:      list of dict, kwargs for scene.graph.update
         base_frame: str, base frame of graph
         """
-        graph = kwargs.get('graph', None)
-        geometry = {k: load_kwargs(v) for
-                    k, v in kwargs['geometry'].items()}
+        graph = kwargs.get("graph", None)
+        geometry = {k: load_kwargs(v) for k, v in kwargs["geometry"].items()}
 
         if graph is not None:
             scene = Scene()
@@ -432,18 +413,25 @@ def load_kwargs(*args, **kwargs):
         else:
             scene = Scene(geometry)
 
-        if 'base_frame' in kwargs:
-            scene.graph.base_frame = kwargs['base_frame']
-        metadata = kwargs.get('metadata')
+        # camera, if it exists
+        camera = kwargs.get("camera")
+        if camera:
+            scene.camera = camera
+            scene.camera_transform = kwargs.get("camera_transform")
+
+        if "base_frame" in kwargs:
+            scene.graph.base_frame = kwargs["base_frame"]
+        metadata = kwargs.get("metadata")
         if isinstance(metadata, dict):
-            scene.metadata.update(kwargs['metadata'])
+            scene.metadata.update(kwargs["metadata"])
         elif isinstance(metadata, str):
             # some ways someone might have encoded a string
             # note that these aren't evaluated until we
             # actually call the lambda in the loop
             candidates = [
                 lambda: json.loads(metadata),
-                lambda: json.loads(metadata.replace("'", '"'))]
+                lambda: json.loads(metadata.replace("'", '"')),
+            ]
             for c in candidates:
                 try:
                     scene.metadata.update(c())
@@ -451,7 +439,7 @@ def load_kwargs(*args, **kwargs):
                 except BaseException:
                     pass
         elif metadata is not None:
-            log.warning('unloadable metadata')
+            log.warning("unloadable metadata")
 
         return scene
 
@@ -460,8 +448,7 @@ def load_kwargs(*args, **kwargs):
         Handle the keyword arguments for a Trimesh object
         """
         # if they've been serialized as a dict
-        if (isinstance(kwargs['vertices'], dict) or
-                isinstance(kwargs['faces'], dict)):
+        if isinstance(kwargs["vertices"], dict) or isinstance(kwargs["faces"], dict):
             return Trimesh(**misc.load_dict(kwargs))
         # otherwise just load that puppy
         return Trimesh(**kwargs)
@@ -470,16 +457,16 @@ def load_kwargs(*args, **kwargs):
         """
         Handle an exported mesh.
         """
-        data, file_type = kwargs['data'], kwargs['file_type']
+        data, file_type = kwargs["data"], kwargs["file_type"]
         if not isinstance(data, dict):
             data = util.wrap_as_stream(data)
-        k = mesh_loaders[file_type](
-            data, file_type=file_type)
+        k = mesh_loaders[file_type](data, file_type=file_type)
         return Trimesh(**k)
 
     def handle_path():
         from ..path import Path2D, Path3D
-        shape = np.shape(kwargs['vertices'])
+
+        shape = np.shape(kwargs["vertices"])
         if len(shape) < 2:
             return Path2D()
         if shape[1] == 2:
@@ -487,26 +474,25 @@ def load_kwargs(*args, **kwargs):
         elif shape[1] == 3:
             return Path3D(**kwargs)
         else:
-            raise ValueError('Vertices must be 2D or 3D!')
+            raise ValueError("Vertices must be 2D or 3D!")
 
     def handle_pointcloud():
         return PointCloud(**kwargs)
 
     # if we've been passed a single dict instead of kwargs
     # substitute the dict for kwargs
-    if (len(kwargs) == 0 and
-        len(args) == 1 and
-            isinstance(args[0], dict)):
+    if len(kwargs) == 0 and len(args) == 1 and isinstance(args[0], dict):
         kwargs = args[0]
 
     # (function, tuple of expected keys)
     # order is important
     handlers = (
-        (handle_scene, ('geometry',)),
-        (handle_mesh, ('vertices', 'faces')),
-        (handle_path, ('entities', 'vertices')),
-        (handle_pointcloud, ('vertices',)),
-        (handle_export, ('file_type', 'data')))
+        (handle_scene, ("geometry",)),
+        (handle_mesh, ("vertices", "faces")),
+        (handle_path, ("entities", "vertices")),
+        (handle_pointcloud, ("vertices",)),
+        (handle_export, ("file_type", "data")),
+    )
 
     # filter out keys with a value of None
     kwargs = {k: v for k, v in kwargs.items() if v is not None}
@@ -518,15 +504,17 @@ def load_kwargs(*args, **kwargs):
             # exit the loop as we found one
             break
     else:
-        raise ValueError('unable to determine type: {}'.format(kwargs.keys()))
+        raise ValueError(f"unable to determine type: {kwargs.keys()}")
 
     return handler()
 
 
-def parse_file_args(file_obj,
-                    file_type,
-                    resolver=None,
-                    **kwargs):
+def _parse_file_args(
+    file_obj: Loadable,
+    file_type: Optional[str],
+    resolver: Union[None, Dict, resolvers.Resolver] = None,
+    **kwargs,
+):
     """
     Given a file_obj and a file_type try to magically convert
     arguments to a file-like object and a lowercase string of
@@ -578,17 +566,16 @@ def parse_file_args(file_obj,
     """
     metadata = {}
     opened = False
-    if ('metadata' in kwargs and
-            isinstance(kwargs['metadata'], dict)):
-        metadata.update(kwargs['metadata'])
+    if "metadata" in kwargs and isinstance(kwargs["metadata"], dict):
+        metadata.update(kwargs["metadata"])
 
     if util.is_pathlib(file_obj):
         # convert pathlib objects to string
         file_obj = str(file_obj.absolute())
 
     if util.is_file(file_obj) and file_type is None:
-        raise ValueError('file_type must be set for file objects!')
-    if util.is_string(file_obj):
+        raise ValueError("file_type must be set for file objects!")
+    if isinstance(file_obj, str):
         try:
             # os.path.isfile will return False incorrectly
             # if we don't give it an absolute path
@@ -604,38 +591,35 @@ def parse_file_args(file_obj,
             if resolver is None:
                 resolver = resolvers.FilePathResolver(file_path)
             # save the file name and path to metadata
-            metadata['file_path'] = file_path
-            metadata['file_name'] = os.path.basename(file_obj)
+            metadata["file_path"] = file_path
+            metadata["file_name"] = os.path.basename(file_obj)
             # if file_obj is a path that exists use extension as file_type
             if file_type is None:
-                file_type = util.split_extension(
-                    file_path,
-                    special=['tar.gz', 'tar.bz2'])
+                file_type = util.split_extension(file_path, special=["tar.gz", "tar.bz2"])
             # actually open the file
-            file_obj = open(file_path, 'rb')
+            file_obj = open(file_path, "rb")
             opened = True
         else:
-            if '{' in file_obj:
+            if "{" in file_obj:
                 # if a dict bracket is in the string, its probably a straight
                 # JSON
-                file_type = 'json'
-            elif 'https://' in file_obj or 'http://' in file_obj:
+                file_type = "json"
+            elif "https://" in file_obj or "http://" in file_obj:
                 # we've been passed a URL, warn to use explicit function
                 # and don't do network calls via magical pipeline
-                raise ValueError(
-                    'use load_remote to load URL: {}'.format(file_obj))
+                raise ValueError(f"use load_remote to load URL: {file_obj}")
             elif file_type is None:
-                raise ValueError('string is not a file: {}'.format(file_obj))
+                raise ValueError(f"string is not a file: {file_obj}")
 
     if file_type is None:
         file_type = file_obj.__class__.__name__
 
-    if util.is_string(file_type) and '.' in file_type:
+    if isinstance(file_type, str) and "." in file_type:
         # if someone has passed the whole filename as the file_type
         # use the file extension as the file_type
-        if 'file_path' not in metadata:
-            metadata['file_path'] = file_type
-        metadata['file_name'] = os.path.basename(file_type)
+        if "file_path" not in metadata:
+            metadata["file_path"] = file_type
+        metadata["file_name"] = os.path.basename(file_type)
         file_type = util.split_extension(file_type)
         if resolver is None and os.path.exists(file_type):
             resolver = resolvers.FilePathResolver(file_type)
@@ -644,25 +628,29 @@ def parse_file_args(file_obj,
     file_type = file_type.lower()
 
     # if we still have no resolver try using file_obj name
-    if (resolver is None and
-        hasattr(file_obj, 'name') and
-        file_obj.name is not None and
-            len(file_obj.name) > 0):
+    if (
+        resolver is None
+        and hasattr(file_obj, "name")
+        and file_obj.name is not None
+        and len(file_obj.name) > 0
+    ):
         resolver = resolvers.FilePathResolver(file_obj.name)
 
     return file_obj, file_type, metadata, opened, resolver
 
 
 # loader functions for compressed extensions
-compressed_loaders = {'zip': load_compressed,
-                      'tar.bz2': load_compressed,
-                      'tar.gz': load_compressed}
+compressed_loaders = {
+    "zip": load_compressed,
+    "tar.bz2": load_compressed,
+    "tar.gz": load_compressed,
+    "bz2": load_compressed,
+}
 
 # map file_type to loader function
 mesh_loaders = {}
 mesh_loaders.update(_misc_loaders)
 mesh_loaders.update(_stl_loaders)
-mesh_loaders.update(_ctm_loaders)
 mesh_loaders.update(_ply_loaders)
 mesh_loaders.update(_obj_loaders)
 mesh_loaders.update(_off_loaders)
@@ -672,6 +660,7 @@ mesh_loaders.update(_xaml_loaders)
 mesh_loaders.update(_threedxml_loaders)
 mesh_loaders.update(_three_loaders)
 mesh_loaders.update(_xyz_loaders)
+mesh_loaders.update(_cascade_loaders)
 
 # collect loaders which return voxel types
 voxel_loaders = {}
