@@ -1,11 +1,11 @@
 import os
 
+from ... import util
+from ...exchange.ply import load_ply
+from ..path import Path
+from . import misc
 from .dxf import _dxf_loaders
 from .svg_io import svg_to_path
-from ..path import Path
-
-from . import misc
-from ... import util
 
 
 def load_path(file_obj, file_type=None, **kwargs):
@@ -31,6 +31,11 @@ def load_path(file_obj, file_type=None, **kwargs):
     path : Path, Path2D, Path3D file_object
         Data as a native trimesh Path file_object
     """
+    # avoid a circular import
+    from ...exchange.load import load_kwargs
+
+    # record how long we took
+    tic = util.now()
 
     if isinstance(file_obj, Path):
         # we have been passed a Path file_object so
@@ -38,33 +43,41 @@ def load_path(file_obj, file_type=None, **kwargs):
         return file_obj
     elif util.is_file(file_obj):
         # for open file file_objects use loaders
-        kwargs.update(path_loaders[file_type](
-            file_obj, file_type=file_type))
-    elif util.is_string(file_obj):
+        if file_type == "ply":
+            # we cannot register this exporter to path_loaders since this is already reserved by TriMesh in ply format in trimesh.load()
+            kwargs.update(load_ply(file_obj, file_type=file_type))
+        else:
+            kwargs.update(path_loaders[file_type](file_obj, file_type=file_type))
+    elif isinstance(file_obj, str):
         # strings passed are evaluated as file file_objects
-        with open(file_obj, 'rb') as f:
+        with open(file_obj, "rb") as f:
             # get the file type from the extension
             file_type = os.path.splitext(file_obj)[-1][1:].lower()
-            # call the loader
-            kwargs.update(path_loaders[file_type](f, file_type=file_type))
-    elif util.is_instance_named(file_obj, ['Polygon', 'MultiPolygon']):
+            if file_type == "ply":
+                # we cannot register this exporter to path_loaders since this is already reserved by TriMesh in ply format in trimesh.load()
+                kwargs.update(load_ply(f, file_type=file_type))
+            else:
+                # call the loader
+                kwargs.update(path_loaders[file_type](f, file_type=file_type))
+    elif util.is_instance_named(file_obj, ["Polygon", "MultiPolygon"]):
         # convert from shapely polygons to Path2D
         kwargs.update(misc.polygon_to_path(file_obj))
-    elif util.is_instance_named(file_obj, 'MultiLineString'):
+    elif util.is_instance_named(file_obj, "MultiLineString"):
         # convert from shapely LineStrings to Path2D
         kwargs.update(misc.linestrings_to_path(file_obj))
     elif isinstance(file_obj, dict):
         # load as kwargs
-        from ...exchange.load import load_kwargs
         return load_kwargs(file_obj)
     elif util.is_sequence(file_obj):
         # load as lines in space
         kwargs.update(misc.lines_to_path(file_obj))
     else:
-        raise ValueError('Not a supported object type!')
+        raise ValueError("Not a supported object type!")
 
-    from ...exchange.load import load_kwargs
-    return load_kwargs(kwargs)
+    result = load_kwargs(kwargs)
+    util.log.debug(f"loaded {result!s} in {util.now() - tic:0.4f}s")
+
+    return result
 
 
 def path_formats():
@@ -80,5 +93,5 @@ def path_formats():
     return set(path_loaders.keys())
 
 
-path_loaders = {'svg': svg_to_path}
+path_loaders = {"svg": svg_to_path}
 path_loaders.update(_dxf_loaders)
