@@ -7,28 +7,25 @@ Subclasses of Trimesh objects that are parameterized as primitives.
 Useful because you can move boxes and spheres around
 and then use trimesh operations on them at any point.
 """
-import numpy as np
+
 import abc
 
-from . import util
-from . import sample
-from . import caching
-from . import inertia
-from . import creation
-from . import triangles
-from . import transformations as tf
+import numpy as np
 
+from . import caching, creation, inertia, sample, triangles, util
+from . import transformations as tf
 from .base import Trimesh
 from .constants import log, tol
+from .typed import ArrayLike, Integer, Number, Optional
 
 # immutable identity matrix for checks
 _IDENTITY = np.eye(4)
 _IDENTITY.flags.writeable = False
 
 
-class _Primitive(Trimesh):
+class Primitive(Trimesh):
     """
-    Geometric _Primitives which are a subclass of Trimesh.
+    Geometric Primitives which are a subclass of Trimesh.
     Mesh is generated lazily when vertices or faces are requested.
     """
 
@@ -36,49 +33,53 @@ class _Primitive(Trimesh):
     __copy__ = None
     __deepcopy__ = None
 
-    def __init__(self, *args, **kwargs):
-        super(_Primitive, self).__init__(*args, **kwargs)
+    def __init__(self):
+        # run the Trimesh constructor with no arguments
+        super().__init__()
+
+        # remove any data
         self._data.clear()
         self._validate = False
+
         # make sure any cached numpy arrays have
         # set `array.flags.writable = False`
         self._cache.force_immutable = True
 
     def __repr__(self):
-        return '<trimesh.primitives.{}>'.format(
-            type(self).__name__)
+        return f"<trimesh.primitives.{type(self).__name__}>"
 
     @property
     def faces(self):
-        stored = self._cache['faces']
+        stored = self._cache["faces"]
         if util.is_shape(stored, (-1, 3)):
             return stored
         self._create_mesh()
-        return self._cache['faces']
+        return self._cache["faces"]
 
     @faces.setter
     def faces(self, values):
-        log.warning('primitive faces are immutable: not setting!')
+        if values is not None:
+            raise ValueError("primitive faces are immutable: not setting!")
 
     @property
     def vertices(self):
-        stored = self._cache['vertices']
+        stored = self._cache["vertices"]
         if util.is_shape(stored, (-1, 3)):
             return stored
 
         self._create_mesh()
-        return self._cache['vertices']
+        return self._cache["vertices"]
 
     @vertices.setter
     def vertices(self, values):
         if values is not None:
-            log.warning('primitive vertices are immutable: not setting!')
+            raise ValueError("primitive vertices are immutable: not setting!")
 
     @property
     def face_normals(self):
         # we need to avoid the logic in the superclass that
         # is specific to the data model prioritizing faces
-        stored = self._cache['face_normals']
+        stored = self._cache["face_normals"]
         if util.is_shape(stored, (-1, 3)):
             return stored
         # just calculate if not stored
@@ -86,13 +87,13 @@ class _Primitive(Trimesh):
         normals = np.zeros((len(valid), 3))
         normals[valid] = unit
         # store and return
-        self._cache['face_normals'] = normals
+        self._cache["face_normals"] = normals
         return normals
 
     @face_normals.setter
     def face_normals(self, values):
         if values is not None:
-            log.warning('Primitive face normals are immutable!')
+            log.warning("Primitive face normals are immutable!")
 
     @property
     def transform(self):
@@ -113,7 +114,7 @@ class _Primitive(Trimesh):
         """
         raise NotImplementedError()
 
-    def copy(self, **kwargs):
+    def copy(self, include_visual=True, **kwargs):
         """
         Return a copy of the Primitive object.
 
@@ -125,14 +126,32 @@ class _Primitive(Trimesh):
         # get the constructor arguments
         kwargs.update(self.to_dict())
         # remove the type indicator, i.e. `Cylinder`
-        kwargs.pop('kind')
+        kwargs.pop("kind")
         # create a new object with kwargs
-        return type(self)(**kwargs)
+        primitive_copy = type(self)(**kwargs)
 
-    def to_mesh(self):
+        if include_visual:
+            # copy visual information
+            primitive_copy.visual = self.visual.copy()
+
+        # copy metadata
+        primitive_copy.metadata = self.metadata.copy()
+
+        for k, v in self._data.data.items():
+            if k not in primitive_copy._data:
+                primitive_copy._data[k] = v
+
+        return primitive_copy
+
+    def to_mesh(self, **kwargs):
         """
         Return a copy of the Primitive object as a Trimesh.
 
+        Parameters
+        -----------
+        kwargs : dict
+          Passed to the Trimesh object constructor.
+
         Returns
         ------------
         mesh : trimesh.Trimesh
@@ -142,7 +161,9 @@ class _Primitive(Trimesh):
             vertices=self.vertices.copy(),
             faces=self.faces.copy(),
             face_normals=self.face_normals.copy(),
-            process=False)
+            process=kwargs.pop("process", False),
+            **kwargs,
+        )
         return result
 
     def apply_transform(self, matrix):
@@ -158,13 +179,12 @@ class _Primitive(Trimesh):
         matrix: (4, 4) float
           Homogeneous transformation
         """
-        matrix = np.asanyarray(
-            matrix, order='C', dtype=np.float64)
+        matrix = np.asanyarray(matrix, order="C", dtype=np.float64)
         if matrix.shape != (4, 4):
-            raise ValueError('matrix must be `(4, 4)`!')
+            raise ValueError("matrix must be `(4, 4)`!")
         if util.allclose(matrix, _IDENTITY, 1e-8):
             # identity matrix is a no-op
-            return
+            return self
 
         prim = self.primitive
         # copy the current transform
@@ -177,19 +197,16 @@ class _Primitive(Trimesh):
         kinds = (Box, Cylinder, Capsule, Sphere)
         if isinstance(self, kinds) and abs(scale - 1.0) > 1e-8:
             # scale the primitive attributes
-            if hasattr(prim, 'height'):
+            if hasattr(prim, "height"):
                 prim.height *= scale
-            if hasattr(prim, 'radius'):
+            if hasattr(prim, "radius"):
                 prim.radius *= scale
-            if hasattr(prim, 'extents'):
+            if hasattr(prim, "extents"):
                 prim.extents *= scale
             # scale the translation of the current matrix
             current[:3, 3] *= scale
             # apply new matrix, rescale, translate, current
-            updated = util.multi_dot([
-                matrix,
-                tf.scale_matrix(1.0 / scale),
-                current])
+            updated = util.multi_dot([matrix, tf.scale_matrix(1.0 / scale), current])
         else:
             # without scaling just multiply
             updated = np.dot(matrix, current)
@@ -204,21 +221,21 @@ class _Primitive(Trimesh):
         return self
 
     def _create_mesh(self):
-        raise ValueError('Primitive doesn\'t define mesh creation!')
+        raise ValueError("Primitive doesn't define mesh creation!")
 
 
-class _PrimitiveAttributes(object):
+class PrimitiveAttributes:
     """
     Hold the mutable data which defines a primitive.
     """
 
-    def __init__(self, parent, defaults, kwargs):
+    def __init__(self, parent, defaults, kwargs, mutable=True):
         """
         Hold the attributes for a Primitive.
 
         Parameters
         ------------
-        parent : _Primitive
+        parent : Primitive
           Parent object reference.
         defaults : dict
           The default values for this primitive type.
@@ -234,7 +251,7 @@ class _PrimitiveAttributes(object):
         # start with a copy of all default objects
         self._data.update(defaults)
         # store whether this data is mutable after creation
-        self._mutable = kwargs.get('mutable', True)
+        self._mutable = mutable
         # assign the keys passed by the user only if
         # they are a property of this primitive
         for key, default in defaults.items():
@@ -252,65 +269,59 @@ class _PrimitiveAttributes(object):
         # operation can be surprisingly slow and most
         # people never call it
         import pprint
+
         doc = (
-            'Store the attributes of a {name} object.\n\n' +
-            'When these values are changed, the mesh geometry will \n' +
-            'automatically be updated to reflect the new values.\n\n' +
-            'Available properties and their default values are:\n {defaults}' +
-            '\n\nExample\n---------------\n' +
-            'p = trimesh.primitives.{name}()\n' +
-            'p.primitive.radius = 10\n' +
-            '\n').format(
+            "Store the attributes of a {name} object.\n\n"
+            + "When these values are changed, the mesh geometry will \n"
+            + "automatically be updated to reflect the new values.\n\n"
+            + "Available properties and their default values are:\n {defaults}"
+            + "\n\nExample\n---------------\n"
+            + "p = trimesh.primitives.{name}()\n"
+            + "p.primitive.radius = 10\n"
+            + "\n"
+        ).format(
             name=self._parent.__class__.__name__,
-            defaults=pprint.pformat(
-                self._defaults,
-                width=-1)[1:-1])
+            defaults=pprint.pformat(self._defaults, width=-1)[1:-1],
+        )
         return doc
 
     def __getattr__(self, key):
-        if key.startswith('_'):
-            return super(_PrimitiveAttributes, self).__getattr__(key)
-        elif key == 'center':
+        if key.startswith("_"):
+            return super().__getattr__(key)
+        elif key == "center":
             # this whole __getattr__ is a little hacky
-            return self._data['transform'][:3, 3]
+            return self._data["transform"][:3, 3]
         elif key in self._defaults:
-            return util.convert_like(self._data[key],
-                                     self._defaults[key])
-        raise AttributeError(
-            "primitive object has no attribute '{}' ".format(key))
+            return util.convert_like(self._data[key], self._defaults[key])
+        raise AttributeError(f"primitive object has no attribute '{key}' ")
 
     def __setattr__(self, key, value):
-        if key.startswith('_'):
-            return super(_PrimitiveAttributes,
-                         self).__setattr__(key, value)
-        elif key == 'center':
+        if key.startswith("_"):
+            return super().__setattr__(key, value)
+        elif key == "center":
             value = np.array(value, dtype=np.float64)
             transform = np.eye(4)
             transform[:3, 3] = value
-            self._data['transform'] = transform
+            self._data["transform"] = transform
             return
         elif key in self._defaults:
             if self._mutable:
-                self._data[key] = util.convert_like(
-                    value, self._defaults[key])
+                self._data[key] = util.convert_like(value, self._defaults[key])
             else:
                 raise ValueError(
-                    'Primitive is configured as immutable! Cannot set attribute!')
+                    "Primitive is configured as immutable! Cannot set attribute!"
+                )
         else:
             keys = list(self._defaults.keys())
-            raise ValueError(
-                'Only default attributes {} can be set!'.format(keys))
+            raise ValueError(f"Only default attributes {keys} can be set!")
 
     def __dir__(self):
-        result = sorted(dir(type(self)) +
-                        list(self._defaults.keys()))
+        result = sorted(dir(type(self)) + list(self._defaults.keys()))
         return result
 
 
-class Cylinder(_Primitive):
-
-    def __init__(self, radius=1.0, height=1.0,
-                 transform=None, sections=32, **kwargs):
+class Cylinder(Primitive):
+    def __init__(self, radius=1.0, height=1.0, transform=None, sections=32, mutable=True):
         """
         Create a Cylinder Primitive, a subclass of Trimesh.
 
@@ -323,21 +334,24 @@ class Cylinder(_Primitive):
         transform : (4, 4) float
           Homogeneous transformation matrix
         sections : int
-          Number of facets in circle
+          Number of facets in circle.
+        mutable : bool
+          Are extents and transform mutable after creation.
         """
-        super(Cylinder, self).__init__(**kwargs)
+        super().__init__()
 
-        defaults = {'height': 10.0,
-                    'radius': 1.0,
-                    'transform': np.eye(4),
-                    'sections': 32}
-        self.primitive = _PrimitiveAttributes(
+        defaults = {"height": 10.0, "radius": 1.0, "transform": np.eye(4), "sections": 32}
+        self.primitive = PrimitiveAttributes(
             self,
-            defaults,
-            {'height': height,
-             'radius': radius,
-             'transform': transform,
-             'sections': sections})
+            defaults=defaults,
+            kwargs={
+                "height": height,
+                "radius": radius,
+                "transform": transform,
+                "sections": sections,
+            },
+            mutable=mutable,
+        )
 
     @caching.cache_decorator
     def volume(self):
@@ -349,8 +363,7 @@ class Cylinder(_Primitive):
         volume : float
           Volume of the cylinder
         """
-        volume = ((np.pi * self.primitive.radius ** 2) *
-                  self.primitive.height)
+        volume = (np.pi * self.primitive.radius**2) * self.primitive.height
         return volume
 
     @caching.cache_decorator
@@ -368,7 +381,8 @@ class Cylinder(_Primitive):
             mass=self.volume,
             radius=self.primitive.radius,
             height=self.primitive.height,
-            transform=self.primitive.transform)
+            transform=self.primitive.transform,
+        )
         return tensor
 
     @caching.cache_decorator
@@ -398,9 +412,8 @@ class Cylinder(_Primitive):
         half = self.primitive.height / 2.0
         # apply the transform to the Z- aligned segment
         points = np.dot(
-            self.primitive.transform,
-            np.transpose([[0, 0, -half, 1],
-                          [0, 0, half, 1]])).T[:, :3]
+            self.primitive.transform, np.transpose([[0, 0, -half, 1], [0, 0, half, 1]])
+        ).T[:, :3]
         return points
 
     def to_dict(self):
@@ -414,10 +427,12 @@ class Cylinder(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'cylinder',
-                'transform': self.primitive.transform.tolist(),
-                'radius': float(self.primitive.radius),
-                'height': float(self.primitive.height), }
+        return {
+            "kind": "cylinder",
+            "transform": self.primitive.transform.tolist(),
+            "radius": float(self.primitive.radius),
+            "height": float(self.primitive.height),
+        }
 
     def buffer(self, distance):
         """
@@ -439,29 +454,28 @@ class Cylinder(_Primitive):
         buffered = Cylinder(
             height=self.primitive.height + distance * 2,
             radius=self.primitive.radius + distance,
-            transform=self.primitive.transform.copy())
+            transform=self.primitive.transform.copy(),
+        )
         return buffered
 
     def _create_mesh(self):
-        log.debug('creating mesh for Cylinder primitive')
-        mesh = creation.cylinder(radius=self.primitive.radius,
-                                 height=self.primitive.height,
-                                 sections=self.primitive.sections,
-                                 transform=self.primitive.transform)
+        log.debug("creating mesh for Cylinder primitive")
+        mesh = creation.cylinder(
+            radius=self.primitive.radius,
+            height=self.primitive.height,
+            sections=self.primitive.sections,
+            transform=self.primitive.transform,
+        )
 
-        self._cache['vertices'] = mesh.vertices
-        self._cache['faces'] = mesh.faces
-        self._cache['face_normals'] = mesh.face_normals
+        self._cache["vertices"] = mesh.vertices
+        self._cache["faces"] = mesh.faces
+        self._cache["face_normals"] = mesh.face_normals
 
 
-class Capsule(_Primitive):
-
-    def __init__(self,
-                 radius=1.0,
-                 height=10.0,
-                 transform=None,
-                 sections=32,
-                 **kwargs):
+class Capsule(Primitive):
+    def __init__(
+        self, radius=1.0, height=10.0, transform=None, sections=32, mutable=True
+    ):
         """
         Create a Capsule Primitive, a subclass of Trimesh.
 
@@ -475,20 +489,23 @@ class Capsule(_Primitive):
           Transformation matrix
         sections : int
           Number of facets in circle
+        mutable : bool
+          Are extents and transform mutable after creation.
         """
-        super(Capsule, self).__init__(**kwargs)
+        super().__init__()
 
-        defaults = {'height': 1.0,
-                    'radius': 1.0,
-                    'transform': np.eye(4),
-                    'sections': 32}
-        self.primitive = _PrimitiveAttributes(
+        defaults = {"height": 1.0, "radius": 1.0, "transform": np.eye(4), "sections": 32}
+        self.primitive = PrimitiveAttributes(
             self,
-            defaults,
-            {'height': height,
-             'radius': radius,
-             'transform': transform,
-             'sections': sections})
+            defaults=defaults,
+            kwargs={
+                "height": height,
+                "radius": radius,
+                "transform": transform,
+                "sections": sections,
+            },
+            mutable=mutable,
+        )
 
     @property
     def transform(self):
@@ -505,10 +522,12 @@ class Capsule(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'capsule',
-                'transform': self.primitive.transform.tolist(),
-                'height': float(self.primitive.height),
-                'radius': float(self.primitive.radius)}
+        return {
+            "kind": "capsule",
+            "transform": self.primitive.transform.tolist(),
+            "height": float(self.primitive.height),
+            "radius": float(self.primitive.radius),
+        }
 
     @caching.cache_decorator
     def direction(self):
@@ -520,66 +539,69 @@ class Capsule(_Primitive):
         axis : (3,) float
           Vector along the cylinder axis
         """
-        axis = np.dot(self.primitive.transform,
-                      [0, 0, 1, 0])[:3]
+        axis = np.dot(self.primitive.transform, [0, 0, 1, 0])[:3]
         return axis
 
     def _create_mesh(self):
-        log.debug('creating mesh for `Capsule` primitive')
+        log.debug("creating mesh for `Capsule` primitive")
 
-        mesh = creation.capsule(radius=self.primitive.radius,
-                                height=self.primitive.height)
+        mesh = creation.capsule(
+            radius=self.primitive.radius, height=self.primitive.height
+        )
         mesh.apply_transform(self.primitive.transform)
 
-        self._cache['vertices'] = mesh.vertices
-        self._cache['faces'] = mesh.faces
-        self._cache['face_normals'] = mesh.face_normals
+        self._cache["vertices"] = mesh.vertices
+        self._cache["faces"] = mesh.faces
+        self._cache["face_normals"] = mesh.face_normals
 
 
-class Sphere(_Primitive):
-
-    def __init__(self,
-                 radius=1.0,
-                 center=None,
-                 transform=None,
-                 subdivisions=3,
-                 **kwargs):
+class Sphere(Primitive):
+    def __init__(
+        self,
+        radius: Number = 1.0,
+        center: Optional[ArrayLike] = None,
+        transform: Optional[ArrayLike] = None,
+        subdivisions: Integer = 3,
+        mutable: bool = True,
+    ):
         """
         Create a Sphere Primitive, a subclass of Trimesh.
 
         Parameters
         ----------
-        radius : float
+        radius
           Radius of sphere
         center : None or (3,) float
           Center of sphere.
         transform : None or (4, 4) float
           Full homogeneous transform. Pass `center` OR `transform.
-        subdivisions : int
+        subdivisions
           Number of subdivisions for icosphere.
+        mutable
+          Are extents and transform mutable after creation.
         """
-        super(Sphere, self).__init__(**kwargs)
-        defaults = {'radius': 1.0,
-                    'transform': np.eye(4),
-                    'subdivisions': 3}
 
-        constructor = {'radius': float(radius),
-                       'subdivisions': int(subdivisions)}
+        super().__init__()
+
+        constructor = {"radius": float(radius), "subdivisions": int(subdivisions)}
         # center is a helper method for "transform"
         # since a sphere is rotationally symmetric
         if center is not None:
             if transform is not None:
-                raise ValueError(
-                    'only one of `center` and `transform` may be passed!')
+                raise ValueError("only one of `center` and `transform` may be passed!")
             translate = np.eye(4)
             translate[:3, 3] = center
-            constructor['transform'] = translate
+            constructor["transform"] = translate
         elif transform is not None:
-            constructor['transform'] = transform
+            constructor["transform"] = transform
 
         # create the attributes object
-        self.primitive = _PrimitiveAttributes(
-            self, defaults, constructor)
+        self.primitive = PrimitiveAttributes(
+            self,
+            defaults={"radius": 1.0, "transform": np.eye(4), "subdivisions": 3},
+            kwargs=constructor,
+            mutable=mutable,
+        )
 
     @property
     def center(self):
@@ -600,17 +622,23 @@ class Sphere(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'sphere',
-                'transform': self.primitive.transform.tolist(),
-                'radius': float(self.primitive.radius)}
+        return {
+            "kind": "sphere",
+            "transform": self.primitive.transform.tolist(),
+            "radius": float(self.primitive.radius),
+        }
 
     @property
     def bounds(self):
         # no docstring so will inherit Trimesh docstring
         # return exact bounds from primitive center and radius (rather than faces)
         # self.extents will also use this information
-        bounds = np.array([self.primitive.center - self.primitive.radius,
-                           self.primitive.center + self.primitive.radius])
+        bounds = np.array(
+            [
+                self.primitive.center - self.primitive.radius,
+                self.primitive.center + self.primitive.radius,
+            ]
+        )
         return bounds
 
     @property
@@ -631,7 +659,7 @@ class Sphere(_Primitive):
         area: float, surface area of the sphere Primitive
         """
 
-        area = 4.0 * np.pi * (self.primitive.radius ** 2)
+        area = 4.0 * np.pi * (self.primitive.radius**2)
         return area
 
     @caching.cache_decorator
@@ -644,7 +672,7 @@ class Sphere(_Primitive):
         volume: float, volume of the sphere Primitive
         """
 
-        volume = (4.0 * np.pi * (self.primitive.radius ** 3)) / 3.0
+        volume = (4.0 * np.pi * (self.primitive.radius**3)) / 3.0
         return volume
 
     @caching.cache_decorator
@@ -657,45 +685,61 @@ class Sphere(_Primitive):
         tensor: (3, 3) float
           3D inertia tensor.
         """
-        return inertia.sphere_inertia(
-            mass=self.volume,
-            radius=self.primitive.radius)
+        return inertia.sphere_inertia(mass=self.volume, radius=self.primitive.radius)
 
     def _create_mesh(self):
-        log.debug('creating mesh for Sphere primitive')
-        unit = creation.icosphere(subdivisions=self.primitive.subdivisions)
-        unit.vertices *= self.primitive.radius
-        unit.vertices += self.primitive.center
+        log.debug("creating mesh for Sphere primitive")
+        unit = creation.icosphere(
+            subdivisions=self.primitive.subdivisions, radius=self.primitive.radius
+        )
 
-        self._cache['vertices'] = unit.vertices
-        self._cache['faces'] = unit.faces
-        self._cache['face_normals'] = unit.face_normals
+        # apply the center offset here
+        self._cache["vertices"] = unit.vertices + self.primitive.center
+        self._cache["faces"] = unit.faces
+        self._cache["face_normals"] = unit.face_normals
 
 
-class Box(_Primitive):
-
-    def __init__(self,
-                 extents=None,
-                 transform=None,
-                 **kwargs):
+class Box(Primitive):
+    def __init__(self, extents=None, transform=None, bounds=None, mutable=True):
         """
         Create a Box Primitive as a subclass of Trimesh
 
         Parameters
         ----------
-        extents : (3,) float
-          Length of each side of the 3D box
-        transform : (4, 4) float
-          Homogeneous transformation matrix for box center
+        extents : Optional[ndarray] (3,) float
+          Length of each side of the 3D box.
+        transform : Optional[ndarray] (4, 4) float
+          Homogeneous transformation matrix for box center.
+        bounds : Optional[ndarray] (2, 3) float
+          Axis aligned bounding box, if passed extents and
+          transform will be derived from this.
+        mutable : bool
+          Are extents and transform mutable after creation.
         """
-        super(Box, self).__init__(**kwargs)
-        defaults = {'transform': np.eye(4),
-                    'extents': np.ones(3)}
-        self.primitive = _PrimitiveAttributes(
+        super().__init__()
+        defaults = {"transform": np.eye(4), "extents": np.ones(3)}
+
+        if bounds is not None:
+            # validate the multiple forms of input available here
+            if extents is not None or transform is not None:
+                raise ValueError(
+                    "if `bounds` is passed `extents` and `transform` must not be!"
+                )
+            bounds = np.array(bounds, dtype=np.float64)
+            if bounds.shape != (2, 3):
+                raise ValueError("`bounds` must be (2, 3) float")
+            # create extents from AABB
+            extents = np.ptp(bounds, axis=0)
+            # translate to the center of the box
+            transform = np.eye(4)
+            transform[:3, 3] = bounds[0] + extents / 2.0
+
+        self.primitive = PrimitiveAttributes(
             self,
-            defaults,
-            {'extents': extents,
-             'transform': transform})
+            defaults=defaults,
+            kwargs={"extents": extents, "transform": transform},
+            mutable=mutable,
+        )
 
     def to_dict(self):
         """
@@ -708,9 +752,11 @@ class Box(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'box',
-                'transform': self.primitive.transform.tolist(),
-                'extents': self.primitive.extents.tolist()}
+        return {
+            "kind": "box",
+            "transform": self.primitive.transform.tolist(),
+            "extents": self.primitive.extents.tolist(),
+        }
 
     @property
     def transform(self):
@@ -733,7 +779,8 @@ class Box(_Primitive):
         samples = sample.volume_rectangular(
             extents=self.primitive.extents,
             count=count,
-            transform=self.primitive.transform)
+            transform=self.primitive.transform,
+        )
         return samples
 
     def sample_grid(self, count=None, step=None):
@@ -755,23 +802,20 @@ class Box(_Primitive):
           Points inside the box
         """
 
-        if (count is not None and
-                step is not None):
-            raise ValueError('only step OR count can be specified!')
+        if count is not None and step is not None:
+            raise ValueError("only step OR count can be specified!")
 
         # create pre- transform bounds from extents
-        bounds = np.array([-self.primitive.extents,
-                           self.primitive.extents]) * .5
+        bounds = np.array([-self.primitive.extents, self.primitive.extents]) * 0.5
 
         if step is not None:
             grid = util.grid_arange(bounds, step=step)
         elif count is not None:
             grid = util.grid_linspace(bounds, count=count)
         else:
-            raise ValueError('either count or step must be specified!')
+            raise ValueError("either count or step must be specified!")
 
-        transformed = tf.transform_points(
-            grid, matrix=self.primitive.transform)
+        transformed = tf.transform_points(grid, matrix=self.primitive.transform)
         return transformed
 
     @property
@@ -780,8 +824,7 @@ class Box(_Primitive):
         Returns whether or not the current box is rotated at all.
         """
         if util.is_shape(self.primitive.transform, (4, 4)):
-            return not np.allclose(self.primitive.transform[
-                                   0:3, 0:3], np.eye(3))
+            return not np.allclose(self.primitive.transform[0:3, 0:3], np.eye(3))
         else:
             return False
 
@@ -795,17 +838,19 @@ class Box(_Primitive):
         volume : float
           Volume of box.
         """
-        volume = float(np.product(self.primitive.extents))
+        volume = float(np.prod(self.primitive.extents))
         return volume
 
     def _create_mesh(self):
-        log.debug('creating mesh for Box primitive')
-        box = creation.box(extents=self.primitive.extents,
-                           transform=self.primitive.transform)
+        log.debug("creating mesh for Box primitive")
+        box = creation.box(
+            extents=self.primitive.extents, transform=self.primitive.transform
+        )
 
-        self._cache['vertices'] = box.vertices
-        self._cache['faces'] = box.faces
-        self._cache['face_normals'] = box.face_normals
+        self._cache.cache.update(box._cache.cache)
+        self._cache["vertices"] = box.vertices
+        self._cache["faces"] = box.faces
+        self._cache["face_normals"] = box.face_normals
 
     def as_outline(self):
         """
@@ -818,18 +863,15 @@ class Box(_Primitive):
         """
         # do the import in function to keep soft dependency
         from .path.creation import box_outline
+
         # return outline with same size as primitive
         return box_outline(
-            extents=self.primitive.extents,
-            transform=self.primitive.transform)
+            extents=self.primitive.extents, transform=self.primitive.transform
+        )
 
 
-class Extrusion(_Primitive):
-    def __init__(self,
-                 polygon=None,
-                 transform=None,
-                 height=1.0,
-                 **kwargs):
+class Extrusion(Primitive):
+    def __init__(self, polygon=None, transform=None, height=1.0, mutable=True):
         """
         Create an Extrusion primitive, which
         is a subclass of Trimesh.
@@ -842,18 +884,27 @@ class Extrusion(_Primitive):
           Transform to apply after extrusion
         height : float
           Height to extrude polygon by
+        mutable : bool
+          Are extents and transform mutable after creation.
         """
         # do the import here, fail early if Shapely isn't installed
         from shapely.geometry import Point
-        super(Extrusion, self).__init__(**kwargs)
+
+        # run the Trimesh init
+        super().__init__()
         # set default values
-        defaults = {'polygon': Point([0, 0]).buffer(1.0),
-                    'transform': np.eye(4),
-                    'height': 1.0}
-        self.primitive = _PrimitiveAttributes(
-            self, defaults, {'transform': transform,
-                             'polygon': polygon,
-                             'height': height})
+        defaults = {
+            "polygon": Point([0, 0]).buffer(1.0),
+            "transform": np.eye(4),
+            "height": 1.0,
+        }
+
+        self.primitive = PrimitiveAttributes(
+            self,
+            defaults=defaults,
+            kwargs={"transform": transform, "polygon": polygon, "height": height},
+            mutable=mutable,
+        )
 
     @caching.cache_decorator
     def area(self):
@@ -868,8 +919,7 @@ class Extrusion(_Primitive):
           Surface area of 3D extrusion
         """
         # area of the sides of the extrusion
-        area = abs(self.primitive.height *
-                   self.primitive.polygon.length)
+        area = abs(self.primitive.height * self.primitive.polygon.length)
         # area of the two caps of the extrusion
         area += self.primitive.polygon.area * 2
         return area
@@ -886,8 +936,7 @@ class Extrusion(_Primitive):
           Volume of 3D extrusion
         """
         # height may be negative
-        volume = abs(self.primitive.polygon.area *
-                     self.primitive.height)
+        volume = abs(self.primitive.polygon.area * self.primitive.height)
         return volume
 
     @caching.cache_decorator
@@ -903,8 +952,8 @@ class Extrusion(_Primitive):
         """
         # only consider rotation and signed height
         direction = np.dot(
-            self.primitive.transform[:3, :3],
-            [0.0, 0.0, np.sign(self.primitive.height)])
+            self.primitive.transform[:3, :3], [0.0, 0.0, np.sign(self.primitive.height)]
+        )
         return direction
 
     @property
@@ -929,9 +978,9 @@ class Extrusion(_Primitive):
         # no docstring for inheritance
         # calculate OBB using 2D polygon and known axis
         from . import bounds
+
         # find the 2D bounding box using the polygon
-        to_origin, box = bounds.oriented_bounds_2D(
-            self.primitive.polygon.exterior.coords)
+        to_origin, box = bounds.oriented_bounds_2D(self.primitive.polygon.exterior.coords)
         #  3D extents
         extents = np.append(box, abs(self.primitive.height))
         # calculate to_3D transform from 2D obb
@@ -939,9 +988,7 @@ class Extrusion(_Primitive):
         rotation_Z[2, 3] = self.primitive.height / 2.0
         # combine the 2D OBB transformation with the 2D projection transform
         to_3D = np.dot(self.primitive.transform, rotation_Z)
-        obb = Box(transform=to_3D,
-                  extents=extents,
-                  mutable=False)
+        obb = Box(transform=to_3D, extents=extents, mutable=False)
         return obb
 
     def slide(self, distance):
@@ -957,8 +1004,7 @@ class Extrusion(_Primitive):
         distance = float(distance)
         translation = np.eye(4)
         translation[2, 3] = distance
-        new_transform = np.dot(self.primitive.transform.copy(),
-                               translation.copy())
+        new_transform = np.dot(self.primitive.transform.copy(), translation.copy())
         self.primitive.transform = new_transform
 
     def buffer(self, distance, distance_height=None, **kwargs):
@@ -996,7 +1042,8 @@ class Extrusion(_Primitive):
             transform=self.primitive.transform.copy(),
             polygon=self.primitive.polygon.buffer(distance),
             height=height,
-            **kwargs)
+            **kwargs,
+        )
 
         # slide the stock along the axis
         buffered.slide(-np.sign(height) * distance_height)
@@ -1014,23 +1061,26 @@ class Extrusion(_Primitive):
         as_dict : dict
           Serializable data for this primitive.
         """
-        return {'kind': 'extrusion',
-                'polygon': self.primitive.polygon.wkt,
-                'transform': self.primitive.transform.tolist(),
-                'height': float(self.primitive.height)}
+        return {
+            "kind": "extrusion",
+            "polygon": self.primitive.polygon.wkt,
+            "transform": self.primitive.transform.tolist(),
+            "height": float(self.primitive.height),
+        }
 
     def _create_mesh(self):
-        log.debug('creating mesh for Extrusion primitive')
+        log.debug("creating mesh for Extrusion primitive")
         # extrude the polygon along Z
         mesh = creation.extrude_polygon(
             polygon=self.primitive.polygon,
             height=self.primitive.height,
-            transform=self.primitive.transform)
+            transform=self.primitive.transform,
+        )
 
         # check volume here in unit tests
         if tol.strict and mesh.volume < 0.0:
-            raise ValueError('matrix inverted mesh!')
+            raise ValueError("matrix inverted mesh!")
 
         # cache mesh geometry in the primitive
-        self._cache['vertices'] = mesh.vertices
-        self._cache['faces'] = mesh.faces
+        self._cache["vertices"] = mesh.vertices
+        self._cache["faces"] = mesh.faces
