@@ -1,20 +1,14 @@
 import numpy as np
 
-from ..constants import log_time
-from .. import remesh
-from .. import grouping
-from .. import util
+from .. import grouping, remesh, util
 from .. import transformations as tr
-
+from ..constants import log_time
 from . import base
 from . import encoding as enc
 
 
 @log_time
-def voxelize_subdivide(mesh,
-                       pitch,
-                       max_iter=10,
-                       edge_factor=2.0):
+def voxelize_subdivide(mesh, pitch: float, max_iter: int = 10, edge_factor: float = 2.0):
     """
     Voxelize a surface by subdividing a mesh until every edge is
     shorter than: (pitch / edge_factor)
@@ -38,29 +32,23 @@ def voxelize_subdivide(mesh,
 
     if max_iter is None:
         longest_edge = np.linalg.norm(
-            mesh.vertices[mesh.edges[:, 0]] -
-            mesh.vertices[mesh.edges[:, 1]],
-            axis=1).max()
-        max_iter = max(int(np.ceil(np.log2(
-            longest_edge / max_edge))), 0)
+            mesh.vertices[mesh.edges[:, 0]] - mesh.vertices[mesh.edges[:, 1]], axis=1
+        ).max()
+        max_iter = max(int(np.ceil(np.log2(longest_edge / max_edge))), 0)
 
     # get the same mesh sudivided so every edge is shorter
     # than a factor of our pitch
-    v, f, idx = remesh.subdivide_to_size(mesh.vertices,
-                                         mesh.faces,
-                                         max_edge=max_edge,
-                                         max_iter=max_iter,
-                                         return_index=True)
+    v, _f, _idx = remesh.subdivide_to_size(
+        mesh.vertices, mesh.faces, max_edge=max_edge, max_iter=max_iter, return_index=True
+    )
 
     # convert the vertices to their voxel grid position
-    hit = v / pitch
-
     # Provided edge_factor > 1 and max_iter is large enough, this is
     # sufficient to preserve 6-connectivity at the level of voxels.
-    hit = np.round(hit).astype(int)
+    hit = np.round(v / pitch).astype(int)
 
     # remove duplicates
-    unique, inverse = grouping.unique_rows(hit)
+    unique, _inverse = grouping.unique_rows(hit)
 
     # get the voxel centers in model space
     occupied_index = hit[unique]
@@ -70,16 +58,11 @@ def voxelize_subdivide(mesh,
 
     return base.VoxelGrid(
         enc.SparseBinaryEncoding(occupied_index - origin_index),
-        transform=tr.scale_and_translate(
-            scale=pitch, translate=origin_position))
+        transform=tr.scale_and_translate(scale=pitch, translate=origin_position),
+    )
 
 
-def local_voxelize(mesh,
-                   point,
-                   pitch,
-                   radius,
-                   fill=True,
-                   **kwargs):
+def local_voxelize(mesh, point, pitch, radius, fill=True, **kwargs):
     """
     Voxelize a mesh in the region of a cube around a point. When fill=True,
     uses proximity.contains to fill the resulting voxels so may be meaningless
@@ -110,11 +93,12 @@ def local_voxelize(mesh,
     # this is a gotcha- radius sounds a lot like it should be in
     # float model space, not int voxel space so check
     if not isinstance(radius, int):
-        raise ValueError('radius needs to be an integer number of cubes!')
+        raise ValueError("radius needs to be an integer number of cubes!")
 
     # Bounds of region
-    bounds = np.concatenate((point - (radius + 0.5) * pitch,
-                             point + (radius + 0.5) * pitch))
+    bounds = np.concatenate(
+        (point - (radius + 0.5) * pitch, point + (radius + 0.5) * pitch)
+    )
 
     # faces that intersect axis aligned bounding box
     faces = list(mesh.triangles_tree.intersection(bounds))
@@ -140,14 +124,15 @@ def local_voxelize(mesh,
     prepad = np.maximum(radius - center, 0)
     postpad = np.maximum(center + radius + 1 - matrix.shape, 0)
 
-    matrix = np.pad(matrix, np.stack((prepad, postpad), axis=-1),
-                    mode='constant')
+    matrix = np.pad(matrix, np.stack((prepad, postpad), axis=-1), mode="constant")
     center += prepad
 
     # Extract voxels within the bounding box
-    voxels = matrix[center[0] - radius:center[0] + radius + 1,
-                    center[1] - radius:center[1] + radius + 1,
-                    center[2] - radius:center[2] + radius + 1]
+    voxels = matrix[
+        center[0] - radius : center[0] + radius + 1,
+        center[1] - radius : center[1] + radius + 1,
+        center[2] - radius : center[2] + radius + 1,
+    ]
     local_origin = point - radius * pitch  # origin of local voxels
 
     # Fill internal regions
@@ -155,26 +140,19 @@ def local_voxelize(mesh,
         regions, n = ndimage.label(~voxels)
         distance = ndimage.distance_transform_cdt(~voxels)
         representatives = [
-            np.unravel_index((distance * (regions == i)).argmax(),
-                             distance.shape) for i in range(1, n + 1)]
-        contains = mesh.contains(
-            np.asarray(representatives) *
-            pitch +
-            local_origin)
-
+            np.unravel_index((distance * (regions == i)).argmax(), distance.shape)
+            for i in range(1, n + 1)
+        ]
+        contains = mesh.contains(np.asarray(representatives) * pitch + local_origin)
         where = np.where(contains)[0] + 1
-        # use in1d vs isin for older numpy versions
-        internal = np.in1d(regions.flatten(), where).reshape(regions.shape)
-
+        internal = np.isin(regions.flatten(), where).reshape(regions.shape)
         voxels = np.logical_or(voxels, internal)
 
     return base.VoxelGrid(voxels, tr.translation_matrix(local_origin))
 
 
 @log_time
-def voxelize_ray(mesh,
-                 pitch,
-                 per_cell=None):
+def voxelize_ray(mesh, pitch, per_cell=None):
     """
     Voxelize a mesh using ray queries.
 
@@ -228,13 +206,12 @@ def voxelize_ray(mesh,
     encoding = enc.SparseBinaryEncoding(voxels)
     origin_position = origin_index * pitch
     return base.VoxelGrid(
-        encoding,
-        tr.scale_and_translate(scale=pitch, translate=origin_position))
+        encoding, tr.scale_and_translate(scale=pitch, translate=origin_position)
+    )
 
 
 @log_time
-def voxelize_binvox(
-        mesh, pitch=None, dimension=None, bounds=None, **binvoxer_kwargs):
+def voxelize_binvox(mesh, pitch=None, dimension=None, bounds=None, **binvoxer_kwargs):
     """
     Voxelize via binvox tool.
 
@@ -273,21 +250,20 @@ def voxelize_binvox(
             extents = maxs - mins
         dimension = int(np.ceil(np.max(extents) / pitch))
     if bounds is not None:
-        if 'bounding_box' in binvoxer_kwargs:
-            raise ValueError('Cannot provide both bounds and bounding_box')
-        binvoxer_kwargs['bounding_box'] = np.asanyarray(bounds).flatten()
+        if "bounding_box" in binvoxer_kwargs:
+            raise ValueError("Cannot provide both bounds and bounding_box")
+        binvoxer_kwargs["bounding_box"] = np.asanyarray(bounds).flatten()
 
     binvoxer = binvox.Binvoxer(dimension=dimension, **binvoxer_kwargs)
     return binvox.voxelize_mesh(mesh, binvoxer)
 
 
 voxelizers = util.FunctionRegistry(
-    ray=voxelize_ray,
-    subdivide=voxelize_subdivide,
-    binvox=voxelize_binvox)
+    ray=voxelize_ray, subdivide=voxelize_subdivide, binvox=voxelize_binvox
+)
 
 
-def voxelize(mesh, pitch, method='subdivide', **kwargs):
+def voxelize(mesh, pitch, method="subdivide", **kwargs):
     """
     Voxelize the given mesh using the specified implementation.
 
