@@ -40,7 +40,9 @@ handled carefully. For example, the `uint8` encoding of 300 zeros
 
 This module contains implementations of various RLE/BRLE operations.
 """
+
 import functools
+
 import numpy as np
 
 
@@ -84,8 +86,7 @@ def rle_to_brle(rle, dtype=None):
     for value, count in np.reshape(rle, (-1, 2)):
         acc += count
         if value not in (0, 1):
-            raise ValueError(
-                "Invalid run length encoding for conversion to BRLE")
+            raise ValueError("Invalid run length encoding for conversion to BRLE")
         if value == curr_val:
             out[-1] += count
         else:
@@ -116,7 +117,7 @@ def brle_logical_not(brle):
     element-wise not of the input.
     """
     if brle[0] or brle[-1]:
-        return np.pad(brle, [1, 1], mode='constant')
+        return np.pad(brle, [1, 1], mode="constant")
     else:
         return brle[1:-1]
 
@@ -162,8 +163,11 @@ def split_long_brle_lengths(lengths, dtype=np.int64):
         remainders = (lengths % max_val).astype(dtype)
 
         lengths = np.concatenate(
-            [np.array([max_val, 0] * repeat + [remainder], dtype=dtype)
-             for repeat, remainder in zip(repeats, remainders)])
+            [
+                np.array([max_val, 0] * repeat + [remainder], dtype=dtype)
+                for repeat, remainder in zip(repeats, remainders)
+            ]
+        )
         lengths = lengths.reshape((np.sum(repeats) * 2 + nl,)).astype(dtype)
         return lengths
     elif lengths.dtype != dtype:
@@ -198,7 +202,7 @@ def dense_to_brle(dense_data, dtype=np.int64):
     lengths = np.diff(np.r_[starts, n])
     lengths = split_long_brle_lengths(lengths, dtype=dtype)
     if dense_data[0]:
-        lengths = np.pad(lengths, [1, 0], mode='constant')
+        lengths = np.pad(lengths, [1, 0], mode="constant")
     return lengths
 
 
@@ -227,10 +231,9 @@ def brle_to_dense(brle_data, vals=None):
     else:
         vals = np.asarray(vals)
         if vals.shape != (2,):
-            raise ValueError("vals.shape must be (2,), got %s" % (vals.shape))
-    ft = np.repeat(
-        _ft[np.newaxis, :], (len(brle_data) + 1) // 2, axis=0).flatten()
-    return np.repeat(ft[:len(brle_data)], brle_data).flatten()
+            raise ValueError(f"vals.shape must be (2,), got {vals.shape}")
+    ft = np.repeat(_ft[np.newaxis, :], (len(brle_data) + 1) // 2, axis=0).flatten()
+    return np.repeat(ft[: len(brle_data)], brle_data).flatten()
 
 
 def rle_to_dense(rle_data, dtype=np.int64):
@@ -239,12 +242,13 @@ def rle_to_dense(rle_data, dtype=np.int64):
     if dtype is not None:
         values = np.asanyarray(values, dtype=dtype)
     try:
-        result = np.repeat(np.squeeze(values, axis=-1),
-                           np.squeeze(counts, axis=-1))
+        result = np.repeat(np.squeeze(values, axis=-1), np.squeeze(counts, axis=-1))
     except TypeError:
         # on windows it sometimes fails to cast data type
-        result = np.repeat(np.squeeze(values.astype(np.int64), axis=-1),
-                           np.squeeze(counts.astype(np.int64), axis=-1))
+        result = np.repeat(
+            np.squeeze(values.astype(np.int64), axis=-1),
+            np.squeeze(counts.astype(np.int64), axis=-1),
+        )
     return result
 
 
@@ -286,7 +290,7 @@ def split_long_rle_lengths(values, lengths, dtype=np.int64):
         repeats += 1
         remainder = lengths % max_length
         values = np.repeat(values, repeats)
-        lengths = np.empty(len(repeats), dtype=dtype)
+        lengths = np.zeros(len(repeats), dtype=dtype)
         lengths.fill(max_length)
         lengths = np.repeat(lengths, repeats)
         lengths[np.cumsum(repeats) - 1] = remainder
@@ -317,8 +321,7 @@ def brle_to_rle(brle, dtype=np.int64):
         brle = np.concatenate([brle, [0]])
     lengths = brle
     values = np.tile(_ft, len(brle) // 2)
-    return rle_to_rle(
-        np.stack((values, lengths), axis=1).flatten(), dtype=dtype)
+    return rle_to_rle(np.stack((values, lengths), axis=1).flatten(), dtype=dtype)
 
 
 def brle_to_brle(brle, dtype=np.int64):
@@ -349,9 +352,7 @@ def _unsorted_gatherer(indices, sorted_gather_fn):
     ordered_indices = indices[order]
 
     def f(data, dtype=None):
-        result = np.empty(
-            len(order), dtype=dtype or getattr(
-                data, 'dtype', None))
+        result = np.zeros(len(order), dtype=dtype or getattr(data, "dtype", None))
         result[order] = tuple(sorted_gather_fn(data, ordered_indices))
         return result
 
@@ -389,8 +390,8 @@ def sorted_rle_gather_1d(rle_data, ordered_indices):
                 start += next(data_iter)
             except StopIteration:
                 raise IndexError(
-                    'Index %d out of range of raw_values length %d'
-                    % (index, start))
+                    "Index %d out of range of raw_values length %d" % (index, start)
+                )
         try:
             while index < start:
                 yield value
@@ -532,8 +533,8 @@ def sorted_brle_gather_1d(brle_data, ordered_indices):
                 start += next(data_iter)
             except StopIteration:
                 raise IndexError(
-                    'Index %d out of range of raw_values length %d'
-                    % (index, start))
+                    "Index %d out of range of raw_values length %d" % (index, start)
+                )
         try:
             while index < start:
                 yield value
@@ -563,7 +564,8 @@ def brle_gatherer_1d(indices):
     or rle_data.dtype if no dtype is provided.
     """
     return functools.partial(
-        _unsorted_gatherer(indices, sorted_brle_gather_1d), dtype=bool)
+        _unsorted_gatherer(indices, sorted_brle_gather_1d), dtype=bool
+    )
 
 
 def brle_gather_1d(brle_data, indices):
@@ -614,7 +616,7 @@ def rle_to_sparse(rle_data):
     try:
         while True:
             value = next(it)
-            counts = next(it)
+            counts = int(next(it))
             end = index + counts
             if value:
                 indices.append(np.arange(index, end, dtype=np.int64))
@@ -627,14 +629,13 @@ def rle_to_sparse(rle_data):
         return indices, values
 
     indices = np.concatenate(indices)
-    values = np.concatenate(values)
+    values = np.concatenate(values, dtype=rle_data.dtype)
     return indices, values
 
 
 def brle_to_sparse(brle_data, dtype=np.int64):
     ends = np.cumsum(brle_data)
-    indices = [np.arange(s, e, dtype=dtype) for s, e in
-               zip(ends[::2], ends[1::2])]
+    indices = [np.arange(s, e, dtype=dtype) for s, e in zip(ends[::2], ends[1::2])]
     return np.concatenate(indices)
 
 
@@ -654,20 +655,24 @@ def rle_strip(rle_data):
     """
     rle_data = np.reshape(rle_data, (-1, 2))
     start = 0
+    final_i = len(rle_data)
     for i, (val, count) in enumerate(rle_data):
         if val and count > 0:
+            final_i = i
             break
         else:
             start += count
 
     end = 0
+    final_j = len(rle_data)
     for j, (val, count) in enumerate(rle_data[::-1]):
         if val and count > 0:
+            final_j = j
             break
         else:
             end += count
 
-    rle_data = rle_data[i:None if j == 0 else -j].reshape((-1,))
+    rle_data = rle_data[final_i : None if final_j == 0 else -final_j].reshape((-1,))
     return rle_data, (start, end)
 
 
@@ -687,20 +692,25 @@ def brle_strip(brle_data):
     """
     start = 0
     val = True
+    final_i = len(brle_data)
     for i, count in enumerate(brle_data):
         val = not val
         if val and count > 0:
+            final_i = i
             break
         else:
             start += count
     end = 0
+    final_j = len(brle_data)
     val = bool(len(brle_data) % 2)
     for j, count in enumerate(brle_data[::-1]):
         val = not val
         if val and count > 0:
+            final_j = j
             break
         else:
             end += count
-    brle_data = brle_data[i:None if j == 0 else -j]
+
+    brle_data = brle_data[final_i : None if final_j == 0 else -final_j]
     brle_data = np.concatenate([[0], brle_data])
     return brle_data, (start, end)
